'
' QBASIC Program
' Run with: QBASIC /RUN SERVOTST.BAS
'
' Author: Ashley Roll. June 2000.
' See http://www.digitalnemesis.com/projects/picservo/ for more information
'
' This program shows how to control the Servo Controller from Quick Basic
' It simply opens the com port (COM1) and echos any data it receives from
' the controller to the console.
'
' It then enables a servo, sets its offset and position and disables it to
' show how to control a servo output. Then it sets a digital output high
' and then low again to show how to do that.
'
'

DECLARE SUB SetDigitalOutputOff (OutputNum%)
DECLARE SUB SetDigitalOutputOn (OutputNum%)
DECLARE SUB EnableServo (ServoNum%)
DECLARE SUB DisableServo (ServoNum%)
DECLARE SUB SetServoPosition (ServoNum%, Value%)
DECLARE SUB SetServoOffset (ServoNum%, Value%)
DECLARE SUB ResetController ()
DECLARE SUB SendCommandToPIC (Cmdstr$)
DECLARE SUB CloseComPort ()
DECLARE SUB OpenComPort (Name$)
DECLARE SUB WaitForKey ()

' Start the program at the right place
GOTO Main

' Com Event Handler. This is called when the serial port
' receives data. Because the PIC only sends strings terminated
' with a CR we can use the LINE INPUT to read them and print them
ComHandler:
	DO WHILE NOT EOF(1)
	PRINT INPUT$(1, 1);
	LOOP
	RETURN

' Main Program start
Main:
	CLS
	PRINT "PIC Serial Port Servo Controller"
	PRINT "Test and Demonstration Program"
	PRINT ""
	PRINT "By Ashley Roll http://www.digitalnemesis.com/projects/picservo/"
	PRINT ""

	' Open the serial port
	PRINT "Opening serial port"
	OpenComPort "COM1:"
      
	WaitForKey

	PRINT "Resetting the Controller"
	ResetController
       
	WaitForKey

	PRINT "Enabling Servo 0"
	EnableServo 0

	WaitForKey
     
	PRINT "Changing Offset for Servo 0"
	SetServoOffset 0, 200

	WaitForKey
    
	PRINT "Changing Position for Servo 0"
	SetServoPosition 0, 80

	WaitForKey
     
	PRINT "Disabling Servo 0"
	DisableServo 0

	WaitForKey
      
	PRINT "Setting Digital output 0 On"
	SetDigitalOutputOn 0

	WaitForKey
      
	PRINT "Setting Digital output 0 Off"
	SetDigitalOutputOff 0

	WaitForKey
       
	PRINT "Exiting"


	' Close the com port
	CloseComPort

SUB CloseComPort
' Close the serial port and remove the incoming data handler
	COM(1) OFF
	CLOSE #1
END SUB

SUB DisableServo (ServoNum%)
' Disable the output of servo control pulses for  the given servo
	' check the servo number
	IF ServoNum% < 0 OR ServoNum% > 3 THEN
		PRINT "Servo Number out of Range. Expected 0-3, got "; SernoNum%
		RETURN
	END IF

	' Send the command to the Controller
	SendCommandToPIC CHR$(64 + ServoNum%)
END SUB

SUB EnableServo (ServoNum%)
' Enable the output of servo control pulses for  the given servo
	' check the servo number
	IF ServoNum% < 0 OR ServoNum% > 3 THEN
		PRINT "Servo Number out of Range. Expected 0-3, got "; SernoNum%
		RETURN
	END IF

	' Send the command to the Controller
	SendCommandToPIC CHR$(48 + ServoNum%)
END SUB

SUB OpenComPort (Name$)
' Subroutine to open the com port into file #1 so that
' the rest of the program may use it to communicate with the
' PIC based servo controller.
'
' The PIC needs to use 2400 baud, no parity, eight data bits
' one stop bit and hardware flow control (uses CTS to control
' when the host may send data

	OPEN Name$ + " 2400,N,8,1" FOR RANDOM AS #1

	' instal a handler for incoming data
	ON COM(1) GOSUB ComHandler
	COM(1) ON
END SUB

SUB ResetController
' Reset the PIC Based Servo Controller
'
' The command for this is 3 consecutive zero bytes
	SendCommandToPIC CHR$(0) + CHR$(0) + CHR$(0)
END SUB

SUB SendCommandToPIC (Cmdstr$)
' Send a command string to the PIC based servo controller
' note that we don't want to append a new line so we use
' the semicolon modifier
	PRINT #1, Cmdstr$;
END SUB

SUB SetDigitalOutputOff (OutputNum%)
' Set the given digital output off (low). OutputNum% is between 0 and 3
	' Check the value of OutputNum%
	IF OutputNum% < 0 OR OutputNum% > 3 THEN
		PRINT "Output Number out of Range. Expected 0-3, got "; OutputNum%
		RETURN
	END IF

	' Send the command to the PIC
	SendCommandToPIC CHR$(96 + OutputNum%)
END SUB

SUB SetDigitalOutputOn (OutputNum%)
' Set the given digital output on (high). OutputNum% is between 0 and 3
	' Check the value of OutputNum%
	IF OutputNum% < 0 OR OutputNum% > 3 THEN
		PRINT "Output Number out of Range. Expected 0-3, got "; OutputNum%
		RETURN
	END IF

	' Send the command to the PIC
	SendCommandToPIC CHR$(80 + OutputNum%)
END SUB

SUB SetServoOffset (ServoNum%, Value%)
' Set the offset value (between 0 and 255) for the given servo
' which ranges between 0 and 3
	' check the servo number
	IF ServoNum% < 0 OR ServoNum% > 3 THEN
		PRINT "Servo Number out of Range. Expected 0-3, got "; SernoNum%
		RETURN
	END IF

	' check the data value is in range
	IF Value% < 0 OR Value% > 255 THEN
		PRINT "Value out of Range. Expected 0-255, got "; Value%
		RETURN
	END IF

	' Send the command to the PIC
	SendCommandToPIC CHR$(32 + ServoNum%) + CHR$(Value%)
END SUB

SUB SetServoPosition (ServoNum%, Value%)
' Set the position value (between 0 and 255) for the given servo
' which ranges between 0 and 3
	' check the servo number
	IF ServoNum% < 0 OR ServoNum% > 3 THEN
		PRINT "Servo Number out of Range. Expected 0-3, got "; SernoNum%
		RETURN
	END IF

	' check the data value is in range
	IF Value% < 0 OR Value% > 255 THEN
		PRINT "Value out of Range. Expected 0-255, got "; Value%
		RETURN
	END IF

	' Send the command to the PIC
	SendCommandToPIC CHR$(16 + ServoNum%) + CHR$(Value%)
END SUB

SUB WaitForKey
	PRINT "Press Any Key"
	DO
	LOOP UNTIL INKEY$ <> ""
END SUB

